#ifndef USBserial_h_
#define USBserial_h_

#include <inttypes.h>

#include "Print.h"
#include "Stream.h"

extern uint8_t joystick_report_data[13];

class usb_joystick_class
{
public:
    usb_joystick_class()
    {
        manual_mode = 0;
    }
    inline void button(uint8_t button, bool val)
    {
        button--;
        uint8_t mask = (1 << (button & 7));
        if (val)
        {
            if (button < 8)
                joystick_report_data[0] |= mask;
            else if (button < 16)
                joystick_report_data[1] |= mask;
            else if (button < 24)
                joystick_report_data[2] |= mask;
            else if (button < 32)
                joystick_report_data[3] |= mask;
        }
        else
        {
            mask = ~mask;
            if (button < 8)
                joystick_report_data[0] &= mask;
            else if (button < 16)
                joystick_report_data[1] &= mask;
            else if (button < 24)
                joystick_report_data[2] &= mask;
            else if (button < 32)
                joystick_report_data[3] &= mask;
        }
        if (!manual_mode)
            send_now();
    }

    inline void X(uint16_t val)
    {
        if (val > 1023)
            val = 1023;
        joystick_report_data[4] = val;
        joystick_report_data[5] = (joystick_report_data[5] & 0xFC) | (val >> 8);
        if (!manual_mode)
            send_now();
    }
    inline void Y(uint16_t val)
    {
        if (val > 1023)
            val = 1023;
        joystick_report_data[5] = (joystick_report_data[5] & 0x03) | (val << 2);
        joystick_report_data[6] = (joystick_report_data[6] & 0xF0) | (val >> 6);
        if (!manual_mode)
            send_now();
    }
    inline void position(uint16_t x, uint16_t y)
    {
        if (x > 1023)
            x = 1023;
        if (y > 1023)
            y = 1023;
        joystick_report_data[4] = x;
        joystick_report_data[5] = (x >> 8) | (y << 2);
        joystick_report_data[6] = (joystick_report_data[6] & 0xF0) | (y >> 6);
        if (!manual_mode)
            send_now();
    }
    inline void Z(uint16_t val)
    {
        if (val > 1023)
            val = 1023;
        joystick_report_data[6] = (joystick_report_data[6] & 0x0F) | (val << 4);
        joystick_report_data[7] = (joystick_report_data[7] & 0xC0) | (val >> 4);
        if (!manual_mode)
            send_now();
    }
    inline void Xrotate(uint16_t val)
    {
        if (val > 1023)
            val = 1023;
        joystick_report_data[7] = (joystick_report_data[7] & 0x3F) | (val << 6);
        joystick_report_data[8] = (val >> 2);
        if (!manual_mode)
            send_now();
    }
    inline void Yrotate(uint16_t val)
    {
        if (val > 1023)
            val = 1023;
        joystick_report_data[9] = val;
        joystick_report_data[10] = (joystick_report_data[10] & 0xFC) | (val >> 8);
        if (!manual_mode)
            send_now();
    }

    inline void hat(uint8_t hatnum, int16_t dir)
    {
        uint8_t val = 15;
        if (dir < 0)
            val = 15;
        else if (dir < 23)
            val = 0;
        else if (dir < 68)
            val = 1;
        else if (dir < 113)
            val = 2;
        else if (dir < 158)
            val = 3;
        else if (dir < 203)
            val = 4;
        else if (dir < 245)
            val = 5;
        else if (dir < 293)
            val = 6;
        else if (dir < 338)
            val = 7;
        switch (hatnum)
        {
            case 1:
                joystick_report_data[10] = (joystick_report_data[10] & 0x0F) | (val << 4);
                break;
            case 2:
                joystick_report_data[11] = (joystick_report_data[11] & 0xF0) | val;
                break;
            case 3:
                joystick_report_data[11] = (joystick_report_data[11] & 0x0F) | (val << 4);
                break;
            case 4:
                joystick_report_data[12] = (joystick_report_data[12] & 0xF0) | val;
                break;
            default:
                break;
        }
        if (!manual_mode)
            send_now();
    }
    inline void useManualSend(bool mode)
    {
        manual_mode = mode;
    }
    uint16_t get_joystick_report_data(uint8_t index);
    void send_now(void);

private:
    uint8_t manual_mode;
};

extern usb_joystick_class Joystick;

extern uint8_t joystick2_report_data[13];

class usb_joystick2_class
{
public:
    usb_joystick2_class()
    {
        manual_mode = 0;
    }
    inline void button(uint8_t button, bool val)
    {
        button--;
        uint8_t mask = (1 << (button & 7));
        if (val)
        {
            if (button < 8)
                joystick2_report_data[0] |= mask;
            else if (button < 16)
                joystick2_report_data[1] |= mask;
            else if (button < 24)
                joystick2_report_data[2] |= mask;
            else if (button < 32)
                joystick2_report_data[3] |= mask;
        }
        else
        {
            mask = ~mask;
            if (button < 8)
                joystick2_report_data[0] &= mask;
            else if (button < 16)
                joystick2_report_data[1] &= mask;
            else if (button < 24)
                joystick2_report_data[2] &= mask;
            else if (button < 32)
                joystick2_report_data[3] &= mask;
        }
        if (!manual_mode)
            send_now();
    }

    inline void X(uint16_t val)
    {
        if (val > 1023)
            val = 1023;
        joystick2_report_data[4] = val;
        joystick2_report_data[5] = (joystick2_report_data[5] & 0xFC) | (val >> 8);
        if (!manual_mode)
            send_now();
    }
    inline void Y(uint16_t val)
    {
        if (val > 1023)
            val = 1023;
        joystick2_report_data[5] = (joystick2_report_data[5] & 0x03) | (val << 2);
        joystick2_report_data[6] = (joystick2_report_data[6] & 0xF0) | (val >> 6);
        if (!manual_mode)
            send_now();
    }
    inline void position(uint16_t x, uint16_t y)
    {
        if (x > 1023)
            x = 1023;
        if (y > 1023)
            y = 1023;
        joystick2_report_data[4] = x;
        joystick2_report_data[5] = (x >> 8) | (y << 2);
        joystick2_report_data[6] = (joystick2_report_data[6] & 0xF0) | (y >> 6);
        if (!manual_mode)
            send_now();
    }
    inline void Z(uint16_t val)
    {
        if (val > 1023)
            val = 1023;
        joystick2_report_data[6] = (joystick2_report_data[6] & 0x0F) | (val << 4);
        joystick2_report_data[7] = (joystick2_report_data[7] & 0xC0) | (val >> 4);
        if (!manual_mode)
            send_now();
    }
    inline void Xrotate(uint16_t val)
    {
        if (val > 1023)
            val = 1023;
        joystick2_report_data[7] = (joystick2_report_data[7] & 0x3F) | (val << 6);
        joystick2_report_data[8] = (val >> 2);
        if (!manual_mode)
            send_now();
    }
    inline void Yrotate(uint16_t val)
    {
        if (val > 1023)
            val = 1023;
        joystick2_report_data[9] = val;
        joystick2_report_data[10] = (joystick2_report_data[10] & 0xFC) | (val >> 8);
        if (!manual_mode)
            send_now();
    }

    inline void hat(uint8_t hatnum, int16_t dir)
    {
        uint8_t val = 15;
        if (dir < 0)
            val = 15;
        else if (dir < 23)
            val = 0;
        else if (dir < 68)
            val = 1;
        else if (dir < 113)
            val = 2;
        else if (dir < 158)
            val = 3;
        else if (dir < 203)
            val = 4;
        else if (dir < 245)
            val = 5;
        else if (dir < 293)
            val = 6;
        else if (dir < 338)
            val = 7;
        switch (hatnum)
        {
            case 1:
                joystick2_report_data[10] = (joystick2_report_data[10] & 0x0F) | (val << 4);
                break;
            case 2:
                joystick2_report_data[11] = (joystick2_report_data[11] & 0xF0) | val;
                break;
            case 3:
                joystick2_report_data[11] = (joystick2_report_data[11] & 0x0F) | (val << 4);
                break;
            case 4:
                joystick2_report_data[12] = (joystick2_report_data[12] & 0xF0) | val;
                break;
            default:
                break;
        }
        if (!manual_mode)
            send_now();
    }
    inline void useManualSend(bool mode)
    {
        manual_mode = mode;
    }
    uint16_t get_joystick2_report_data(uint8_t index);
    void send_now(void);

private:
    uint8_t manual_mode;
};

extern usb_joystick2_class Joystick2;


class usb_serial_class : public Stream
{
public:
    // standard Arduino functions
    void begin(long);
    void end();
    virtual int available();
    virtual int read();
    virtual int peek();
    virtual void flush();
    virtual size_t write(uint8_t);
    using Print::write;
    operator bool();
    // Teensy extensions
    void send_now(void);
    uint32_t baud(void);
    uint8_t stopbits(void);
    uint8_t paritytype(void);
    uint8_t numbits(void);
    uint8_t dtr(void);
    uint8_t rts(void);

private:
    uint8_t readnext(void);
};

extern usb_serial_class Serial;

#endif
