#ifndef BUTTON_MATRIX_H
#define BUTTON_MATRIX_H

#include "Button.h"

#ifndef INPUT_PULLUP
#warning "Using  pinMode() INPUT_PULLUP AVR emulation"
#define INPUT_PULLUP 0x2
#define pinMode(_pin, _mode) _mypinMode(_pin, _mode)
#define _mypinMode(_pin, _mode)  \
  do {							 \
    if(_mode == INPUT_PULLUP)	 \
      pinMode(_pin, INPUT);	 \
    digitalWrite(_pin, 1);	 \
    if(_mode != INPUT_PULLUP)	 \
      pinMode(_pin, _mode);	 \
  }while(0)
#endif


#define OPEN LOW
#define CLOSED HIGH

typedef int ButtonEvent;
typedef unsigned int uint;
typedef unsigned long ulong;

typedef struct {
  byte rows;
  byte columns;
} ButtonMatrixSize;

#define LIST_MAX 100		// Max number of buttons on the active list.
#define MAPSIZE 10		// MAPSIZE is the number of rows (times 16 columns)

//class ButtonMatrix : public Button, public HAL_obj {
class ButtonMatrix : public Button {
  public:

    ButtonMatrix(byte *row, byte *col, byte numRows, byte numCols);

    virtual void pin_mode(byte pinNum, byte mode) {
      pinMode(pinNum, mode);
    }
    virtual void pin_write(byte pinNum, boolean level) {
      digitalWrite(pinNum, level);
    }
    virtual int  pin_read(byte pinNum) {
      return digitalRead(pinNum);
    }

    uint bitMap[MAPSIZE];	// 10 row x 16 column array of bits. Except Due which has 32 columns.
    Button button[LIST_MAX];
    unsigned long holdTimer;

    int getButton();
    bool getButtons();
    ButtonState getState();
    bool isPressed(int buttonIndex);
    void setDebounceTime(uint);
    void setHoldTime(uint);
    void addEventListener(void (*listener)(int));
    int findInList(int buttonIndex);
    int waitForButton();
    bool buttonStateChanged();
    byte numButtons();

  private:
    unsigned long startTime;
    byte *rowPins;
    byte *columnPins;
    byte numRows;
    byte numCols;
    ButtonMatrixSize sizeBM;
    uint debounceTime;
    uint holdTime;
    bool single_button;

    void scanButtons();
    bool updateList();
    void nextButtonState(byte n, boolean bbutton);
    void transitionTo(byte n, ButtonState nextState);
    void (*buttonEventListener)(int);
};

#endif
